// transform.js

// 'regexp-data' action handler
function regexpData(node,action){
	if(node!=null){
		if(node.nodeType==3){
  	  // Get the pattern
  	  var patternStr = getFirstChildTextValue(action,"pattern");
  	  // Extract the javascript pattern and modifier
      var e = /\/?([^\/]*)\/?([g,i,m]{1,2})?/;
  	  if(patternStr.match(e)){
    	  var pattern = RegExp.$1;
    	  var modifier = RegExp.$2;
    	  // Get the substitution
    	  var substitution = getFirstChildTextValue(action,"substitution");
  			var oldText = node.data;
  			// Build a new Regexp object
  			var reg = new RegExp(pattern,modifier);
  			var index = oldText.search(reg);
  			if(index != -1){
  			  // Replace the text
  			  var newText = oldText.replace(reg,substitution);
  				// Update the text node
  				node.data = newText;
  			}
			}
		}
	}
}

// 'set-attr' action handler
function setAttrib(node,action){
  // Get the attribute name
  var name = getFirstChildTextValue(action,"name");
  // Get the attribute value
  var value = getFirstChildTextValue(action,"value");
  node.setAttribute(name,value);
}

// Select target nodes of a doc for a given XPATH
function selectNodes(doc,path){
   return doc.evaluate(path,
                       doc,
                       null,
                       XPathResult.UNORDERED_NODE_SNAPSHOT_TYPE,
                       null);
}

// Perform Kaizou actions on target doc starting at a given node 
function performActions(target,node,actions){
  for(var i=0,n=actions.length;i<n;i++){
    var action = actions[i];
    if(action.nodeType==1){
  	  // Get the action type
  	  var type = action.localName.toLowerCase();
  	  switch(type){
  	   case "comment":
        commentText(target,node,action); // in comments.js
        break;
       case "regexp-data":
        regexpData(node,action);
        break;
       case "set-attr":
        setAttrib(node,action);
        break;
       default:
        break;
      } 
    } 
  }
}

// Apply transformations described in doc to target 
function applyTransformations(target,doc){
  try{
    // Go to the transformation node
    var transformations = doc.getElementsByTagName("transform");
    if(transformations){
  		for(var i=0,n=transformations.length;i<n;i++){
  		  // Get the path
  		  var path = getFirstChildTextValue(transformations[i],"xpath");  
        // Get the actions
        var actions = getFirstChild(transformations[i],"actions");
        if(actions){		  
    		  // Select nodes
          var targetNodes = selectNodes(target,path);
          var j=0;
          while ( (node = targetNodes.snapshotItem(j) ) !=null ){
            // Perform actions on the node
            performActions(target,node,actions.childNodes);
            j++;
          }
        }    		  
  		}
    }  
  }
  catch(e){
    alert(e);
  }
}

////////////////////////////////////////////////////////////////////////////////
// Kaizou editor functions (for future use). 
// They use today the old kaizou format (no xpath)
////////////////////////////////////////////////////////////////////////////////

function createChildNode(doc,node,name){
  var child = doc.createElement(name);
  node.appendChild(child);
}

function createChildNodeWithText(doc,node,name,value){
  var child = doc.createElement(name);
  var text = doc.createTextNode(value);
  child.appendChild(text);
  node.appendChild(child);
}


function addTransformation(doc,type,pattern,substitution){
  try{
    // Go to the transform node
    var transform = doc.getElementsByTagName("transform");
    if(transform && (transform.length>0)){
      var action = doc.createElement("action");
      action.setAttribute("type",type);
      var patNode = doc.createElement("pattern");
      var patText = doc.createTextNode(pattern);
      patNode.appendChild(patText);
      action.appendChild(patNode);
      var subNode = doc.createElement("substitution");
      var subText = doc.createTextNode(substitution);
      subNode.appendChild(subText);
      action.appendChild(subNode);
      transform[0].appendChild(action);
    }
  }
  catch(e)
  {
    alert(e);
  }
}

function newKaizouDescriptionDocument(){
  // Create document
  var doc = document.implementation.createDocument("http://www.kaizou.org","kaizou",null);
  // Add Title, Author & Link children
  createChildNodeWithText(doc,doc.firstChild,"title","");
  createChildNodeWithText(doc,doc.firstChild,"author","");
  createChildNodeWithText(doc,doc.firstChild,"link","");
  // Add target & transform children
  createChildNode(doc,doc.firstChild,"target");
  createChildNode(doc,doc.firstChild,"transform");
  return doc;
}
